@@ -18,6 +18,9 @@
 ##' original order.
 ##' @param sumstat If TRUE (default), include mean, standard deviation, and count of NAs.
 ##' @param digits integer, used for number formatting output.
+##' @param na.rm default TRUE. Should missing data be removed?
+##' @param unbiased If TRUE (default), skewness and kurtosis are calculated with
+##' biased corrected (N-1) divisor in the standard devation.
 ##' @export
 ##' @return a matrix with one column per variable and the rows
 ##' representing the quantiles as well as the mean, standard
@@ -25,7 +28,7 @@
 ##' @seealso summarize and summarizeFactors
 ##' @author Paul E. Johnson <pauljohn@@ku.edu>
 summarizeNumerics <- function(dat, alphaSort = TRUE, sumstat = TRUE,
-    digits = max(3, getOption("digits") - 3)) {
+    digits = max(3, getOption("digits") - 3), na.rm = TRUE, unbiased = TRUE) {
     if (is.atomic(dat)) {
         datname <- deparse(substitute(dat))
         dat <- data.frame(dat)
@@ -35,26 +38,144 @@ summarizeNumerics <- function(dat, alphaSort = TRUE, sumstat = TRUE,
             colnames(dat) <- paste0(datname, "_",  seq(1, NCOL(dat)))
         }
     } else if (!is.data.frame(dat)) dat <- as.data.frame(dat)
+    var2 <- function(x, na.rm, unbiased) {
+        if (unbiased) {
+            var(x, na.rm = na.rm)
+        } else {
+            mean((x - mean(x, na.rm = na.rm))^2)
+        }
+    }
+
+    sd2 <- function(x, na.rm, unbiased) {
+        if (unbiased) {
+            sd(x, na.rm = na.rm)
+        } else {
+            sqrt(mean((x - mean(x, na.rm = na.rm))^2))
+        }
+    }
+    
     
     nums <- sapply(dat, is.numeric)
     if (sum(nums) == 0) return(NULL)
     datn <- dat[, nums, drop = FALSE]
     if (alphaSort)
         datn <- datn[, sort(colnames(datn)), drop = FALSE]
-    sumdat <- apply(datn, 2, stats::quantile, na.rm = TRUE)
+    sumdat <- apply(datn, 2, stats::quantile, na.rm = na.rm)
     sumdat <- round(sumdat, digits)
     if (sumstat) {
-        sumdat <- rbind(sumdat, mean = apply(datn, 2, mean, na.rm = TRUE))
-        sumdat <- rbind(sumdat, sd = apply(datn, 2, sd, na.rm = TRUE))
-        sumdat <- rbind(sumdat, var = apply(datn, 2, var, na.rm = TRUE))
+        sumdat <- rbind(sumdat, mean = apply(datn, 2, mean, na.rm = na.rm))
+        sumdat <- rbind(sumdat, sd = apply(datn, 2, sd2, na.rm = na.rm, unbiased = unbiased))
+        sumdat <- rbind(sumdat, var = apply(datn, 2, var2, na.rm = na.rm, unbiased = unbiased))
+        sumdat <- rbind(sumdat, skewness = apply(datn, 2, skewness, na.rm = na.rm, unbiased = unbiased))
+        sumdat <- rbind(sumdat, kurtosis = apply(datn, 2, kurtosis, na.rm = na.rm, unbiased = unbiased))
         sumdat <- round(sumdat, digits)
-        sumdat <- rbind(sumdat, `NA's` = apply(datn, 2, function(x) sum(is.na(x))))
-        sumdat <- rbind(sumdat, N = apply(datn, 2, function(x) length(x)))
+        sumdat <- rbind(sumdat, `NA's` = round(apply(datn, 2, function(x) sum(is.na(x))), 0))
+        sumdat <- rbind(sumdat, N = round(apply(datn, 2, function(x) length(x)), 0))
     }
    sumdat
 }
 NULL
 
+##' Calculate excess kurtosis
+##'
+##' Kurtosis is a summary of the peakedness of a distribution. In a Normal distribution,
+##' the kurtosis is 3.  The term "excess kurtosis" refers to the difference
+##'
+##' kurtosis - 3
+##'
+##' Many researchers use the term kurtosis to refer to "excess
+##' kurtosis" and this function follows suit by returning excess
+##' kurtosis.  The user may avoid this by setting excess = FALSE, in
+##' which case kurtosis is returned.
+##' 
+##' If na.rm = FALSE and there are missing values, the mean and
+##' variance are undefined and this function returns NA.
+##' 
+##' The kurtosis may be calculated with the small-sample
+##' bias-corrected estimate of the variance. Set unbiased = FALSE if
+##' this is not desired.  It appears somewhat controversial whether
+##' this is necessary, hence the argument unbiased. According to the
+##' US NIST,
+##' \url{http://www.itl.nist.gov/div898/handbook/eda/section3/eda35b.htm},
+##' kurtosis is defined as
+##'       
+##'                 mean((x - mean(x))^4)
+##' kurtosis =    ___________________
+##'                  var(x)^2
+##'
+##' where var(x) is calculated with the denominator N, rather than N-1.
+##'
+##' A distribution is said to be leptokurtotic if it is tightly bunched in the center (spiked) and there are long, narrow tails representing extreme values that might occur.
+##' @param x A numeric variable (vector)
+##' @param na.rm default TRUE. Should missing data be removed?
+##' @param excess default TRUE. If true, function returns excess kurtosis (kurtosis -3). If false, the return is simply kurtosis as defined above.
+##' @param unbiased default TRUE. Should the denominator of the variance estimate be divided by N-1, rather than N?
+##' @export
+##' @return A scalar value or NA
+##' @author Paul Johnson <pauljohn@@ku.edu>
+kurtosis <- function(x, na.rm = TRUE, excess = TRUE, unbiased = TRUE){
+    if (!isTRUE(na.rm) & sum(is.na(x) > 0)) return(NA)
+    x <- x[!is.na(x)]
+    xm <- mean(x)
+    xd <- x - xm
+    var <- mean(xd^2)
+   
+    if (unbiased){
+        kur <- mean(xd^4)/(var(x)^2)
+    } else {
+         kur <- mean(xd^4)/var^2
+    }
+    if (isTRUE(excess)) kur <- kur - 3
+    kur
+}
+
+
+##' Calculate skewness
+##'
+##' Skewness is a summary of the symmetry of a distribution's
+##' probability density function. In a Normal distribution, the
+##' skewness is 0, indicating symmetry about the expected value.
+##'
+##' If na.rm = FALSE and there are missing values, the mean and
+##' variance are undefined and this function returns NA.
+##'
+##' The skewness may be calculated with the small-sample bias-corrected
+##' estimate of the standard deviation.  It appears somewhat controversial
+##' whether this is necessary, hence the argument unbiased is provided.
+##' Set unbiased = FALSE if it is desired to have the one recommended
+##' by NIST, for example. According to the US NIST,
+##' \url{http://www.itl.nist.gov/div898/handbook/eda/section3/eda35b.htm},
+##' skewness is defined as the mean of cubed deviations divided by the
+##' cube of the standard deviation.
+##'       
+##'                mean((x - mean(x))^3)
+##' skewness =    ___________________
+##'                  sd(x)^3
+##'
+##' where sd(x) is calculated with the denominator N, rather than
+##' N-1. This is the Fisher-Pearson coefficient of skewness, they claim.
+##' The unbiased variant uses the standard deviation divisor (N-1) to
+##' bias-correct the standard deviation. 
+##' 
+##' @param x A numeric variable (vector)
+##' @param na.rm default TRUE. Should missing data be removed?
+##' @param unbiased default TRUE. Should the denominator of the variance estimate be divided by N-1?
+##' @export
+##' @return A scalar value or NA
+##' @author Paul Johnson <pauljohn@@ku.edu>
+skewness <- function(x, na.rm = TRUE, unbiased = TRUE){
+    if (!isTRUE(na.rm) & sum(is.na(x) > 0)) return(NA)
+    x <- x[!is.na(x)]
+    xm <- mean(x)
+    if (unbiased){
+        skew <-  mean((x - xm)^3)/var(x)^(3/2)
+    } else {
+        skew <- mean((x - xm)^3)/(mean((x - xm)^2))^(3/2)
+    }
+    skew
+}
+
+
 
 ##' Extracts non-numeric variables, calculates summary information,
 ##' including entropy as a diversity indicator.
@@ -132,7 +253,7 @@ NULL
 ##' ##see help for summarize for more examples
 summarizeFactors <-
     function (dat = NULL, maxLevels = 5, alphaSort = TRUE,
-              sumstat= TRUE, digits = max(3, getOption("digits") - 3))
+              sumstat = TRUE, digits = max(3, getOption("digits") - 3))
 {
     if (is.atomic(dat)){
         datname <- deparse(substitute(dat))
@@ -219,13 +340,15 @@ NULL
 ##'
 ##' The work is done by the functions \code{summarizeNumerics} and
 ##' \code{summarizeFactors}.  Please see the help pages for those
-##' functions for complete details.
+##' functions for complete details. Named argumes used here must be
+##' spelled fully so they can be sorted and passed to those 2 funcitons.
 ##'
 ##' @param dat A data frame
 ##' @param ... Optional arguments that are passed to summarizeNumerics and summarizeFactors.  These may be used:
-##'  maxLevels The maximum number of levels that will be reported.
-##'  alphaSort If TRUE (default), the columns are re-organized in alphabetical order. If FALSE, they are presented in the original order.
-##'  digits  integer, used for number formatting output.
+##'  1) maxLevels The maximum number of levels that will be reported.
+##'  2) alphaSort If TRUE (default), the columns are re-organized in alphabetical order. If FALSE, they are presented in the original order.
+##'  3) digits  integer, used for number formatting output.
+##'  4) unbiased If TRUE (default), the variance, standard deviation, skewness and kurtosis are based on the (N-1) denominator formula. 
 ##' @return A list with 2 objects, numerics and factors. numerics is a matrix
 ##' of summary information, while factors is a list of factor summaries.
 ##' @export
